/*******************************************************************************

      NN   NN                                    ii                     lll
      NNN  NN                                                            ll
      NNNN NN uu   uu mmm mmm  eeeee  rr rrrr   iii    ccccc   aaaaa     ll
      NN NNNN uu   uu mmmmmmm ee   ee  rrr rr    ii   cc   cc      aa    ll
      NN  NNN uu   uu mm m mm eeeeeee  rr        ii   cc       aaaaaa    ll
      NN   NN uu   uu mm   mm ee       rr        ii   cc   cc aa   aa    ll
      NN   NN  uuuuuu mm   mm  eeeee   rr       iiii   ccccc   aaaaaa   llll

IIII           tt                                     tt    ii
 II            tt                                     tt
 II  nnnnnn  tttttt  eeeee   gggggg rr rrrr  aaaaa  tttttt iii   ooooo  nnnnnn
 II  nn   nn   tt   ee   ee gg   gg  rrr rr      aa   tt    ii  oo   oo nn   nn
 II  nn   nn   tt   eeeeeee gg   gg  rr      aaaaaa   tt    ii  oo   oo nn   nn
 II  nn   nn   tt   ee       gggggg  rr     aa   aa   tt    ii  oo   oo nn   nn
IIII nn   nn   tttt  eeeee       gg  rr      aaaaaa   tttt iiii  ooooo  nn   nn
                              ggggg
                --------------------------------------------------
                   INTEST: test numerical integration methods
             Dudley J. Benton, Knoxville, TN, dudley.benton@gmail.com

-- History of Revisions --------------------------------------------------------

  V1.00 (02/14/1989): original version
  V1.10 (03/03/1989): fix bug in Chebyshev, Gauss, Lobatto abscissas
  V1.20 (07/21/1989): add Romberg
  V2.00 (10/13/1993): translate into C
  V2.10 (05/06/2010): 32-bit compatability
  V2.11 (06/01/2010): increase accuracy of existing and add more Lobatto methods
  V2.12 (06/26/2021): change from coarse (GetTickCount) to fine
                      (QueryPerformanceCounter) Windows timer

*******************************************************************************/

char*Version="2.12";

#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include <stdarg.h>
#include <stddef.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#define _USE_MATH_DEFINES
#include <float.h>
#include <math.h>
//#define WIN32_LEAN_AND_MEAN
//#include <windows.h>

extern int __stdcall QueryPerformanceFrequency(__int64*);
extern int __stdcall QueryPerformanceCounter(__int64*);

__int64 rate,t1,t2;

double Ftest(double X)
  {
  double F;
  F=X*tan(X)+X*atan2(X,1/X)+cos(X)*sin(X)+atan2(1/X,X)+atan2(2*X,3/X);
  if(F==DBL_MAX&&X<0)
    return(F);
  else
    return(1/X);
  }

double Trapz(int n,double F(double),double X1,double X2)  /* Trapezoidal Rule */
  {
  int i;
  double dX,T,X;
  T=0;
  if(n==1)
    T=(X2-X1)*F((X1+X2)/2);
  else if(n==2)
    T=(X2-X1)*(F(X1)+F(X2))/2;
  else if(n>2)
    {
    X=X1;
    dX=(X2-X1)/(n-1);
    T=(F(X1)+F(X2))/2;
    for(i=2;i<n;i++)
      {
      X+=dX;
      T+=F(X);
      }
    T*=dX;
    }
  return(T);
  }

double Simps(double F(double),double X1,double X2,int n)    /* Simpson's Rule */
  {
  int i,k;
  double dX,S,X;
  S=0;
  if(n==3)
    S=(X2-X1)*(F(X1)+4*F((X1+X2)/2)+F(X2))/6;
  else if(n==4)
    S=(X2-X1)*(F(X1)+3*F((3*X1+X2)/4)+3*F((X1+3*X2)/4)+F(X2))/8;
  else if(n>4)
    {
    k=(n-3)/2;
    dX=(X2-X1)/(2*(k+1));
    X=X1+dX;
    S=F(X1)+4*F(X)+F(X2);
    for(i=0;i<k;i++)
      {
      X+=dX;
      S+=2*F(X);
      X+=dX;
      S+=4*F(X);
      }
    S*=dX/3;
    }
  return(S);
  }

double Rombg(double F(double),double X1,double X2,int n)  /* Romberg's Method */
  {
  int i,i1,j,j1,jj;
  double dX,E,H,H2,S,X;
  static double R[15][15];

  dX=X2-X1;
  H=dX;
  R[0][0]=(F(X1)+F(X2))*H/2;
  if(n<2)
    return(R[0][0]);

  jj=1;
  i1=0;
  for(i=1;i<n;i++)
    {
    S=0;
    H2=H/2;
    X=X1-H2;
    for(j=0;j<jj;j++)
      {
      X+=H;
      S+=F(X);
      }
    R[i][0]=(R[i1][0]+H*S)/2;
    H=H2;
    E=1;
    j1=0;
    for(j=1;j<=i;j++)
      {
      E*=4;
      R[i][j]=(E*R[i][j1]-R[i1][j1])/(E-1);
      j1=j;
      }
    jj+=jj;
    i1=i;
    }
  return(R[n-1][n-1]);
  }

double NewC4(double F(double),double X1,double X2)    /* 4-point Newton-Cotes */
  {
  int i,n=4;
  double C,dX,X,W[4]={1,3,3,1};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1)/8);
  }

double NewC5(double F(double),double X1,double X2)    /* 5-point Newton-Cotes */
  {
  int i,n=5;
  double C,dX,X,W[5]={7,32,12,32,7};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1)/90);
  }

double NewC6(double F(double),double X1,double X2)    /* 6-point Newton-Cotes */
  {
  int i,n=6;
  double C,dX,X,W[6]={19,75,50,50,75,19};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1)/288);
  }

double NewC7(double F(double),double X1,double X2)    /* 7-point Newton-Cotes */
  {
  int i,n=7;
  double C,dX,X,W[7]={41,216,27,272,27,216,41};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1)/840);
  }

double NewC8(double F(double),double X1,double X2)    /* 8-point Newton-Cotes */
  {
  int i,n=8;
  double C,dX,X,W[8]={751,3577,1323,2989,2989,1323,3577,751};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1)/17280);
  }

double NewC9(double F(double),double X1,double X2)    /* 9-point Newton-Cotes */
  {
  int i,n=9;
  double C,dX,X,W[9]={989,5888,-928,10496,-4540,10496,-928,5888,989};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1)/28350);
  }

double NewC10(double F(double),double X1,double X2)  /* 10-point Newton-Cotes */
  {
  int i,n=10;
  double C,dX,X,W[10]={2857,15741,1080,19344,5778,5778,19344,1080,15741,2857};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1)/89600);
  }

double NewC11(double F(double),double X1,double X2)  /* 11-point Newton-Cotes */
  {
  int i,n=11;
  double C,dX,X,W[11]={16067,106300,-48525,272400,-260550,427368,-260550,272400,-48525,106300,16067};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1)/598752);
  }

double NewC12(double F(double),double X1,double X2)  /* 12-point Newton-Cotes */
  {
  int i,n=12;
  double C,dX,X;
  static double W[12]={    .0249332309119641, .1548553585207226,
    -.0371692317937990, .2896582547949741,-.1101780891754860,
     .1779004767416227, .1779004767416230,-.1101780891754863,
     .2896582547949741,-.0371692317937991, .1548553585207226,
     .0249332309119641};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1));
  }

double NewC16(double F(double),double X1,double X2)  /* 16-point Newton-Cotes */
  {
  int i,n=16;
  double C,dX,X;
  static double W[16]={    .0170872997716264, .1285073786774053,
    -.1127229050595076, .5070427082102271,-.7562931148441474,
    1.1913603495068070,-.9680052114962560, .4930234952338425,
     .4930234952338438,-.9680052114962570,1.1913603495068090,
    -.7562931148441480, .5070427082102273,-.1127229050595076,
     .1285073786774054, .0170872997716265};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1));
  }

double NewC20(double F(double),double X1,double X2)  /* 20-point Newton-Cotes */
  {
  int i,n=20;
  double C,dX,X;
  static double W[20]={    .0128354111727805, .1122448429762599,
    -.1723394504951573, .7998487183489099,-1.923336545547666,
    4.1532996672365270,-6.569715212385619,8.1667568023714130,
    -6.896320610000664,2.8167263763232190,2.8167263763232100,
    -6.896320610000659,8.1667568023714100,-6.569715212385617,
    4.1532996672365260,-1.923336545547666, .7998487183489098,
    -.1723394504951573, .1122448429762599, .0128354111727805};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1));
  }

double NewC30(double F(double),double X1,double X2)  /* 30-point Newton-Cotes */
  {
  int i,n=30;
  double C,dX,X;
  static double W[30]={    .0077238575608618, .0893727166207986,
    -.2899573364854603,1.8008790811498760,-7.932044796565177,
    29.608058219500310,-91.08562631848949,235.13004988941700,
    -511.0477270492341,938.41924709925770,-1450.817415284024,
    1866.7925467508590,-1938.619179433495,1490.6040306430500,
    -562.1599580391235,-562.1599580391269,1490.6040306430550,
    -1938.619179433499,1866.7925467508620,-1450.817415284025,
    938.41924709925890,-511.0477270492344,235.13004988941710,
    -91.08562631848950,29.608058219500330,-7.932044796565181,
    1.8008790811498770,-.2899573364854605, .0893727166207987,
     .0077238575608618};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1));
  }

double NewC40(double F(double),double X1,double X2)  /* 40-point Newton-Cotes */
  {
  int i,n=40;
  double C,dX,X;
  static double W[40]={    .0054269194001815, .0769684556818667,
    -.3859250056871648,3.1327439645104060,-19.70041736065070,
    104.72424500666330,-469.3194895552246,1797.7904774425630,
    -5943.630584836714,17099.882927488670,-43077.43918902167,
    95438.821262438040,-186422.2454432459,321159.27764825030,
    -486772.8403946455,644719.04972972900,-735279.8257105608,
    698794.40119677720,-506851.9969687242,185720.72149648100,
    185720.72149649480,-506851.9969687372,698794.40119678760,
    -735279.8257105692,644719.04972973500,-486772.8403946487,
    321159.27764825230,-186422.2454432469,95438.821262438530,
    -43077.43918902186,17099.882927488740,-5943.630584836736,
    1797.7904774425680,-469.3194895552259,104.72424500666360,
    -19.70041736065075,3.1327439645104120,-.3859250056871656,
     .0769684556818668, .0054269194001815};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1));
  }

double NewC50(double F(double),double X1,double X2)   /* 50-point Newton-Cotes */
  {
  int i,n=50;
  double C,dX,X;
  static double W[50]={    .0041412907092220, .0689689417280302,
    -.4706848232136333,4.7593484954492430,-38.74486215665367,
    267.21395608455320,-1568.480355687493,7939.1322293714980,
    -35032.57356276532,136023.47603727140,-468294.9785633062,
    1438470.5638076310,-3962249.213073124,9825601.2389776710,
    -22000865.15389250,44570526.918208340,-81770568.16724378,
    135822609.32486040,-203874297.6990139,275379026.03401530,
    -331986012.1228634,351601256.26876890,-316326575.5859812,
    221275895.81571930,-79632117.12894264,-79632117.12893872,
    221275895.81571500,-316326575.5859786,351601256.26876620,
    -331986012.1228615,275379026.03401410,-203874297.6990131,
    135822609.32485990,-81770568.16724358,44570526.918208230,
    -22000865.15389246,9825601.2389776580,-3962249.213073117,
    1438470.5638076280,-468294.9785633053,136023.47603727130,
    -35032.57356276527,7939.1322293714930,-1568.480355687491,
    267.21395608455290,-38.74486215665363,4.7593484954492390,
    -.4706848232136329, .0689689417280302, .0041412907092220};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1));
  }

double NewC60(double F(double),double X1,double X2)  /* 60-point Newton-Cotes */
  {
  int i,n=60;
  double C,dX,X;
  static double W[60]={    .0033267559887382, .0632846354792416,
    -.5483702655184450,6.6588522639863050,-66.46216064187897,
    563.33563722251440,-4085.432123444879,25681.635797746330,
    -141526.5596774685,690382.68822795640,-3005421.166239935,
    11755384.506197820,-41548645.22915461,133332405.54838640,
    -390033294.6779199,1043484158.0706090,-2560079233.666816,
    5771948479.8872750,-11977618996.33722,22899532426.630970,
    -40350125592.52267,65501348490.054140,-97822499295.38669,
    134028124740.32260,-167630239998.4623,189705910462.71180,
    -191111880301.6837,165660891145.38890,-113061683893.1367,
    40191816024.262240,40191816024.271760,-113061683893.1454,
    165660891145.39710,-191111880301.6906,189705910462.71720,
    -167630239998.4661,134028124740.32510,-97822499295.38828,
    65501348490.055090,-40350125592.52320,22899532426.631240,
    -11977618996.33735,5771948479.8873310,-2560079233.666840,
    1043484158.0706190,-390033294.6779229,133332405.54838750,
    -41548645.22915491,11755384.506197900,-3005421.166239955,
    690382.68822795990,-141526.5596774693,25681.635797746460,
    -4085.432123444900,563.33563722251720,-66.46216064187929,
    6.6588522639863360,-.5483702655184478, .0632846354792418,
     .0033267559887382};
  dX=(X2-X1)/(n-1);
  C=0;
  X=X1;
  for(i=0;i<n;i++)
    {
    C+=W[i]*F(X);
    X+=dX;
    }
  return(C*(X2-X1));
  }

double Cheb2(double F(double),double X1,double X2)       /* 2-point Chebyshev */
  {
  double dX,C,aX;
  static double X=.5773502692;
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  C=dX*(F(aX-dX*X)+F(aX+dX*X))/2;
  return(C);
  }

double Cheb3(double F(double),double X1,double X2)       /* 3-point Chebyshev */
  {
  double dX,C,aX;
  static double X=.7071067812;
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  C=dX*(F(aX)+F(aX-dX*X)+F(aX+dX*X))/1.5;
  return(C);
  }

double Cheb4(double F(double),double X1,double X2)       /* 4-point Chebyshev */
  {
  short int i;
  double dX,C,aX;
  static double A[]={.1875924741,.7946544723};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  C=0;
  for(i=0;i<2;i++)
    C+=F(aX-dX*A[i])+F(aX+dX*A[i]);
  C=C*dX/2;
  return(C);
  }

double Cheb5(double F(double),double X1,double X2)       /* 5-point Chebyshev */
  {
  short int i;
  double dX,C,aX;
  static double A[]={.3745414096,.8324974870};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  C=F(aX);
  for(i=0;i<2;i++)
    C+=F(aX-dX*A[i])+F(aX+dX*A[i]);
  C=C*dX/2.5;
  return(C);
  }

double Cheb6(double F(double),double X1,double X2)       /* 6-point Chebyshev */
  {
  short int i;
  double dX,C,aX;
  static double A[]={.2666354015,.4225186538,.8662468181};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  C=0;
  for(i=0;i<3;i++)
    C+=F(aX-dX*A[i])+F(aX+dX*A[i]);
  C=C*dX/3;
  return(C);
  }

double Cheb7(double F(double),double X1,double X2)       /* 7-point Chebyshev */
  {
  short int i;
  double dX,C,aX;
  static double A[]={.3239118105,.5296518105,.8838617008};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  C=F(aX);
  for(i=0;i<3;i++)
    C+=F(aX-dX*A[i])+F(aX+dX*A[i]);
  C=C*dX/3.5;
  return(C);
  }

double Cheb9(double F(double),double X1,double X2)       /* 9-point Chebyshev */
  {
  short int i;
  double dX,C,aX;
  static double A[]={.1679061842,.5287617831,.6010186554,.9115893077};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  C=F(aX);
  for(i=0;i<4;i++)
    C+=F(aX-dX*A[i])+F(aX+dX*A[i]);
  C=C*dX/4.5;
  return(C);
  }

double Loba3(double F(double),double X1,double X2)         /* 3-point Lobatto */
  {
  double dX,L,aX;
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  L=dX*(F(X1)+4*F(aX)+F(X2))/3;
  return(L);
  }

double Loba4(double F(double),double a,double b)           /* 4-point Lobatto */
  {
  double dX,L,aX;
  static double A[]={0.447213595499958,1.000000000000000};
  static double W[]={0.833333333333333,0.166666666666667};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=dX*(W[0]*(F(aX-dX*A[0])+F(aX+dX*A[0]))+W[1]*(F(a)+F(b)));
  return(L);
  }

double Loba5(double F(double),double a,double b)           /* 5-point Lobatto */
  {
  double dX,L,aX;
  static double A[]={0.000000000000000,0.654653670707977,1.000000000000000};
  static double W[]={0.711111111111111,0.544444444444444,0.100000000000000};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=dX*(W[2]*(F(a)+F(b))+W[0]*F(aX)+W[1]*(F(aX-dX*A[1])+F(aX+dX*A[1])));
  return(L);
  }

double Loba6(double F(double),double a,double b)           /* 6-point Lobatto */
  {
  short int i;
  double dX,L,aX;
  static double A[]={0.285231516480645,0.765055323929465,1.000000000000000};
  static double W[]={0.554858377035487,0.378474956297847,0.066666666666667};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=W[2]*(F(a)+F(b));
  for(i=0;i<2;i++)
    L+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  L*=dX;
  return(L);
  }

double Loba7(double F(double),double a,double b)           /* 7-point Lobatto */
  {
  short int i,n=7;
  double dX,L,aX;
  static double A[]={0.000000000000000,0.468848793470714,0.830223896278567,
                     1.000000000000000};
  static double W[]={0.487619047619048,0.431745381209863,0.276826047361566,
                     0.047619047619048};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=W[n/2]*(F(a)+F(b))+W[0]*F(aX);
  for(i=1;i<n/2;i++)
    L+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  L*=dX;
  return(L);
  }

double Loba8(double F(double),double a,double b)           /* 8-point Lobatto */
  {
  short int i,n=8;
  double dX,L,aX;
  static double A[]={0.209299217902479,0.591700181433142,0.871740148509607,
                     1.000000000000000};
  static double W[]={0.412458794658704,0.341122692483504,0.210704227143506,
                     0.035714285714286};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=W[(n-1)/2]*(F(a)+F(b));
  for(i=0;i<(n-1)/2;i++)
    L+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  L*=dX;
  return(L);
  }

double Loba9(double F(double),double a,double b)           /* 9-point Lobatto */
  {
  short int i,n=9;
  double dX,L,aX;
  static double A[]={0.000000000000000,0.363117463826178,0.677186279510738,
                     0.899757995411460,1.000000000000000};
  static double W[]={0.371519274376417,0.346428510973046,0.274538712500162,
                     0.165495361560806,0.027777777777778};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=W[n/2]*(F(a)+F(b))+W[0]*F(aX);
  for(i=1;i<n/2;i++)
    L+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  L*=dX;
  return(L);
  }

double Loba10(double F(double),double a,double b)         /* 10-point Lobatto */
  {
  short int i,n=10;
  double dX,L,aX;
  static double A[]={0.165278957666387,0.477924949810444,0.738773865105505,
                     0.919533908166459,1.000000000000000};
  static double W[]={0.327539761183897,0.292042683679684,0.224889342063126,
                     0.133305990851070,0.022222222222222};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=W[(n-1)/2]*(F(a)+F(b));
  for(i=0;i<(n-1)/2;i++)
    L+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  L*=dX;
  return(L);
  }

double Loba12(double F(double),double a,double b)         /* 12-point Lobatto */
  {
  short int i,n=12;
  double dX,L,aX;
  static double A[]={0.136552932854928,0.399530940965349,0.632876153031861,
                     0.819279321644007,0.944899272222882,1.000000000000000};
  static double W[]={0.271405240910696,0.251275603199201,0.212508417761021,
                     0.157974705564370,0.091684517413196,0.015151515151515};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=W[(n-1)/2]*(F(a)+F(b));
  for(i=0;i<(n-1)/2;i++)
    L+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  L*=dX;
  return(L);
  }

double Loba16(double F(double),double a,double b)         /* 16-point Lobatto */
  {
  short int i,n=16;
  double dX,L,aX;
  static double A[]={0.101326273521949,0.299830468900763,0.486059421887138,
                     0.652388702882493,0.792008291861815,0.899200533093472,
                     0.969568046270218,1.000000000000000};
  static double W[]={0.201958308178230,0.193690023825204,0.177491913391704,
                     0.154026980807164,0.124255382132514,0.089393697325931,
                     0.050850361005920,0.008333333333333};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=W[(n-1)/2]*(F(a)+F(b));
  for(i=0;i<(n-1)/2;i++)
    L+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  L*=dX;
  return(L);
  }

double Loba20(double F(double),double a,double b)         /* 20-point Lobatto */
  {
  short int i,n=20;
  double dX,L,aX;
  static double A[]={0.080545937238822,0.239551705922987,0.392353183713909,
                     0.534992864031886,0.663776402290311,0.775368260952056,
                     0.866877978089950,0.935934498812665,0.980743704893914,
                     1.000000000000000};
  static double W[]={0.160743286387846,0.156580102647476,0.148361554070917,
                     0.136300482358724,0.120709227628675,0.101991499699451,
                     0.080631763996120,0.057181802127567,0.032237123188489,
                     0.005263157894737};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=W[(n-1)/2]*(F(a)+F(b));
  for(i=0;i<(n-1)/2;i++)
    L+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  L*=dX;
  return(L);
  }

double Loba24(double F(double),double a,double b)         /* 24-point Lobatto */
  {
  short int i,n=24;
  double dX,L,aX;
  static double A[]={0.066837993737229,0.199321253390833,0.328247613375511,
                     0.451316373214323,0.566331357979295,0.671240105264129,
                     0.764170482420493,0.843464070154872,0.907705675113506,
                     0.955748220929886,0.986730553505161,1.000000000000000};
  static double W[]={0.133476843866986,0.131094941873604,0.126373642028021,
                     0.119397193702491,0.110290086892969,0.099214827684084,
                     0.086369029967929,0.071981862055294,0.056309848724646,
                     0.039631681333468,0.022236853464711,0.003623188405797};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=W[(n-1)/2]*(F(a)+F(b));
  for(i=0;i<(n-1)/2;i++)
    L+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  L*=dX;
  return(L);
  }

double Loba32(double F(double),double a,double b)         /* 32-point Lobatto */
  {
  short int i,n=32;
  double dX,L,aX;
  static double A[]={0.049864725046593,0.149098596813648,0.246850658850205,
                     0.342149406538881,0.434047717201847,0.521632262881565,
                     0.604032587148421,0.680429755615551,0.750064493936675,
                     0.812244731777442,0.866352476012676,0.911849939063732,
                     0.948284838417232,0.975294690482709,0.992608933972761,
                     1.000000000000000};
  static double W[]={0.099646771501277,0.098656436540762,0.096685608948003,
                     0.093753875546814,0.089890372957358,0.085133497949668,
                     0.079530525692106,0.073137139602679,0.066016877257155,
                     0.058240497248056,0.049885271336221,0.041034201586063,
                     0.031775135410915,0.022199552889292,0.012398106501374,
                     0.002016129032258};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=W[(n-1)/2]*(F(a)+F(b));
  for(i=0;i<(n-1)/2;i++)
    L+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  L*=dX;
  return(L);
  }

double Loba48(double F(double),double a,double b)         /* 48-point Lobatto */
  {
  short int i,n=48;
  double dX,L,aX;
  static double A[]={0.033068864766153,0.099061992550750,0.164621947398091,
                     0.229462052271245,0.293298777849759,0.355852982328906,
                     0.416851132032703,0.476026497502857,0.533120319832777,
                     0.587882942145485,0.640074901265957,0.689467974811091,
                     0.735846179112848,0.779006713599844,0.818760847485606,
                     0.854934744838774,0.887370224318220,0.915925449976246,
                     0.940475549335081,0.960913153506382,0.977148846890837,
                     0.989111470013636,0.996747781339858,1.000000000000000};
  static double W[]={0.066113618696002,0.065824520631008,0.065247588651726,
                     0.064385345531601,0.063241561635038,0.061821238429979,
                     0.060130586616844,0.058176998969275,0.055969018005149,
                     0.053516298628675,0.050829565905945,0.047920568156735,
                     0.044802025564072,0.041487574518847,0.037991707926728,
                     0.034329711700674,0.030517597621304,0.026572032590911,
                     0.022510263736936,0.018350036475219,0.014109490605489,
                     0.009806931978900,0.005459192600248,0.000886524822695};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=W[(n-1)/2]*(F(a)+F(b));
  for(i=0;i<(n-1)/2;i++)
    L+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  L*=dX;
  return(L);
  }

double Loba64(double F(double),double a,double b)         /* 64-point Lobatto */
  {
  short int i,n=64;
  double dX,L,aX;
  static double A[]={0.024736727621959,0.074149647946116,0.123381111650028,
                     0.172310641087793,0.220818497496953,0.268785974019170,
                     0.316095686195626,0.362631859226262,0.408280611289854,
                     0.452930232231581,0.496471456936058,0.538797732716801,
                     0.579805480067718,0.619394346138431,0.657467450312977,
                     0.693931621290705,0.728697625088837,0.761680383408120,
                     0.792799181826208,0.821977867307517,0.849145034542991,
                     0.874234200657628,0.897183967845850,0.917938173510282,
                     0.936446027475634,0.952662235788663,0.966547110369099,
                     0.978066662831396,0.987192676602740,0.993902726703057,
                     0.998179871502163,1.000000000000000};
  static double W[]={0.049463363620777,0.049342318477140,0.049100524407501,
                     0.048738573122233,0.048257350376415,0.047658033802221,
                     0.046942090027028,0.046111271084289,0.045167610125948,
                     0.044113416446892,0.042951269833602,0.041684014250802,
                     0.040314750881560,0.038846830537808,0.037283845459801,
                     0.035629620524489,0.033888203884125,0.032063857057727,
                     0.030161044499089,0.028184422665848,0.026138828614338,
                     0.024029268144024,0.021860903511518,0.019639040723242,
                     0.017369116384542,0.015056683987961,0.012707399197455,
                     0.010327002366815,0.007921289790047,0.005496016203817,
                     0.003056008244912,0.000496031746032};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=W[(n-1)/2]*(F(a)+F(b));
  for(i=0;i<(n-1)/2;i++)
    L+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  L*=dX;
  return(L);
  }

double Loba96(double F(double),double a,double b)         /* 96-point Lobatto */
  {
  short int i,n=96;
  double dX,L,aX;
  static double A[]={0.016448063714370,0.049326393248958,0.082151348359585,
                     0.114887410271099,0.147499156396670,0.179951298667367,
                     0.212208721716186,0.244236520875195,0.276000039944693,
                     0.307464908693509,0.338597080049860,0.369362866942534,
                     0.399728978752520,0.429662557335655,0.459131212577301,
                     0.488103057440580,0.516546742470245,0.544431489714841,
                     0.571727126030472,0.598404115730099,0.624433592543079,
                     0.649787390850332,0.674438076161339,0.698358974800003,
                     0.721524202767228,0.743908693748980,0.765488226239521,
                     0.786239449750429,0.806139910077042,0.825168073594918,
                     0.843303350559982,0.860526117387033,0.876817737882358,
                     0.892160583407209,0.906538051949890,0.919934586085042,
                     0.932335689799266,0.943727944162189,0.954099021820540,
                     0.963437700287926,0.971733873989057,0.978978564977784,
                     0.985163932126614,0.990283278154330,0.994331051990612,
                     0.997302833017008,0.999195175376926,1.000000000000000};
  static double W[]={0.032893160792024,0.032857568213815,0.032786421570920,
                     0.032679797848712,0.032537812421109,0.032360618925734,
                     0.032148409097665,0.031901412561967,0.031619896585223,
                     0.031304165786330,0.030954561806883,0.030571462941493,
                     0.030155283728448,0.029706474501154,0.029225520900838,
                     0.028712943351056,0.028169296494549,0.027595168593089,
                     0.026991180890925,0.026357986942556,0.025696271905530,
                     0.025006751799048,0.024290172729166,0.023547310081440,
                     0.022778967681880,0.021985976927117,0.021169195884722,
                     0.020329508364648,0.019467822962775,0.018585072077598,
                     0.017682210901064,0.016760216384642,0.015820086181631,
                     0.014862837566750,0.013889506333909,0.012901145672925,
                     0.011898825025498,0.010883628919882,0.009856655781517,
                     0.008819016711677,0.007771834211915,0.006716240789720,
                     0.005653377236371,0.004584389784779,0.003510422375025,
                     0.002432578603011,0.001351534905656,0.000219298245614};
  dX=(b-a)/2;
  aX=(a+b)/2;
  L=W[(n-1)/2]*(F(a)+F(b));
  for(i=0;i<(n-1)/2;i++)
    L+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  L*=dX;
  return(L);
  }

double Gaus2(double F(double),double X1,double X2)           /* 2-point Gauss */
  {
  double dX,G,aX;
  static double X=.577350269189626;
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=dX*(F(aX-dX*X)+F(aX+dX*X));
  return(G);
  }

double Gaus3(double F(double),double X1,double X2)           /* 3-point Gauss */
  {
  double dX,G,aX;
  static double A[]={.000000000000000,.774596669241483};
  static double W[]={.888888888888888,.555555555555556};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=dX*(W[0]*F(aX)+W[1]*(F(aX-dX*A[1])+F(aX+dX*A[1])));
  return(G);
  }

double Gaus4(double F(double),double X1,double X2)           /* 4-point Gauss */
  {
  short int i;
  double dX,G,aX;
  static double A[]={.339981043585,.861136311594};
  static double W[]={.652145154863,.347854845137};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=0;
  for(i=0;i<2;i++)
    G+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  G*=dX;
  return(G);
  }

double Gaus5(double F(double),double X1,double X2)           /* 5-point Gauss */
  {
  short int i;
  double dX,G,aX;
  static double A[]={.000000000000,.538469310106,.906179845939};
  static double W[]={.568888888889,.478628670499,.236926885056};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=W[0]*F(aX);
  for(i=1;i<3;i++)
    G+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  G*=dX;
  return(G);
  }

double Gaus6(double F(double),double X1,double X2)           /* 6-point Gauss */
  {
  short int i;
  double dX,G,aX;
  static double A[]={.238619186083,.661209386466,.932469514203};
  static double W[]={.467913934573,.360761573048,.171324492379};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=0;
  for(i=0;i<3;i++)
    G+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  G*=dX;
  return(G);
  }

double Gaus7(double F(double),double X1,double X2)           /* 7-point Gauss */
  {
  short int i;
  double dX,G,aX;
  static double A[]={.000000000000,.405845151377,.741531185599,.949107912343};
  static double W[]={.417959183673,.381830050505,.279705391489,.129484966169};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=W[0]*F(aX);
  for(i=1;i<4;i++)
    G+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  G*=dX;
  return(G);
  }

double Gaus8(double F(double),double X1,double X2)           /* 8-point Gauss */
  {
  short int i;
  double dX,G,aX;
  static double A[]={.183434642496,.525532409916,.796666477414,.960289856498};
  static double W[]={.362683783378,.313706645878,.222381034453,.101228536290};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=0;
  for(i=0;i<4;i++)
    G+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  G*=dX;
  return(G);
  }

double Gaus9(double F(double),double X1,double X2)           /* 9-point Gauss */
  {
  short int i;
  double dX,G,aX;
  static double A[]={.000000000000,.324253423404,.613371432701,.836031107327,.968160239508};
  static double W[]={.330239355001,.312347077040,.260610696403,.180648160695,.081274388362};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=W[0]*F(aX);
  for(i=1;i<5;i++)
    G+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  G*=dX;
  return(G);
  }

double Gaus10(double F(double),double X1,double X2)         /* 10-point Gauss */
  {
  short int i;
  double dX,G,aX;
  static double A[]={.148874338982,.433395394129,.679409568299,.865063366689,.973906528517};
  static double W[]={.295524224715,.269266719310,.219086362516,.149451349151,.066671344309};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=0;
  for(i=0;i<5;i++)
    G+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  G*=dX;
  return(G);
  }

double Gaus12(double F(double),double X1,double X2)         /* 12-point Gauss */
  {
  short int i;
  double dX,G,aX;
  static double A[]={.125233408511,.367831498998,.587317954287,
                  .769902674194,.904117256370,.981560634247};
  static double W[]={.249147045813,.233492536538,.203167426723,
                  .160078328543,.106939325995,.047175336387};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=0;
  for(i=0;i<6;i++)
    G+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  G*=dX;
  return(G);
  }

double Gaus16(double F(double),double X1,double X2)         /* 16-point Gauss */
  {
  short int i;
  double dX,G,aX;
  static double A[]={.095012509838,.281603550797,.458016777657,
                  .617876244403,.755404408355,.865631202388,
                  .944575023073,.989400934992};
  static double W[]={.189450610455,.182603415045,.169156519395,
                  .149595988817,.124628971256,.095158511682,
                  .062253523939,.027152459412};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=0;
  for(i=0;i<8;i++)
    G+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  G*=dX;
  return(G);
  }

double Gaus20(double F(double),double X1,double X2)         /* 20-point Gauss */
  {
  short int i;
  double dX,G,aX;
  static double A[]={.076526521133,.227785851142,.373706088715,
                  .510867001951,.636053680727,.746331906460,
                  .839116971822,.912234428251,.963971927278,
                  .993128599185};
  static double W[]={.152753387131,.149172986473,.142096109318,
                  .131688638449,.118194531962,.101930119817,
                  .083276741577,.062672048334,.040601429800,
                  .017614007139};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=0;
  for(i=0;i<10;i++)
    G+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  G*=dX;
  return(G);
  }

double Gaus40(double F(double),double X1,double X2)         /* 40-point Gauss */
  {
  short int i;
  double dX,G,aX;
  static double A[]={.038772417506,.116084070675,.192697580701,
    .268152185007,.341994090826,.413779204372,.483075801686,
    .549467125095,.612553889668,.671956684614,.727318255190,
    .778305651427,.824612230833,.865959503212,.902098806969,
    .932812808279,.957916819214,.977259949984,.990726238699,
    .998237709711};
  static double W[]={.077505947978,.077039818164,.076110361901,
    .074723169058,.072886582396,.070611647391,.067912045815,
    .064804013457,.061306242493,.057439769099,.053227846984,
    .048695807635,.043870908186,.038782167974,.033460195283,
    .027937006980,.022245849194,.016421058382,.010498284531,
    .004521277099};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=0;
  for(i=0;i<20;i++)
    G+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  G*=dX;
  return(G);
  }

double Gaus96(double F(double),double X1,double X2)         /* 96-point Gauss */
  {
  short int i;
  double dX,G,aX;
  static double A[]={ .016276744849604,.048812985136050,.081297495464426,
  .113695850110666,.145973714654898,.178096882367620,.210031310460568,
  .241743156163840,.273198812591050,.304364944354496,.335208522892624,
  .365696861472314,.395797649828908,.425478988407302,.454709422167742,
  .483457973920596,.511694177154668,.539388108324356,.566510418561398,
  .593032364777572,.618925840125470,.644163403784968,.668718310043916,
  .692564536642172,.715676812348968,.738030643744400,.759602341176646,
  .780369043867432,.800308744139140,.819400310737932,.837623511228188,
  .854959033434602,.871388505909298,.886894517402420,.901460635315852,
  .915071423120898,.927712456722310,.939370339752754,.950032717784438,
  .959688291448742,.968326828463264,.975939174585136,.982517263563014,
  .988054126329624,.992543900323762,.995981842987208,.998364375863182,
  .999689503883230};
  static double W[]={ .032550614492362,.032516118713870,.032447163714064,
  .032343822568576,.032206204794030,.032034456231994,.031828758894412,
  .031589330770728,.031316425596862,.031010332586314,.030671376123668,
  .030299915420828,.029896344136328,.029461089958168,.028994614150556,
  .028497411065086,.027970007616848,.027412962726028,.026826866725592,
  .026212340735672,.025570036005348,.024900633222484,.024204841792366,
  .023483399085926,.022737069658330,.021966644438744,.021172939892190,
  .020356797154332,.019519081140146,.018660679627412,.017782502316044,
  .016885479864246,.015970562902562,.015038721026996,.014090941772316,
  .013128229566962,.012151604671088,.011162102099838,.010160770535008,
  .009148671230784,.008126876925698,.007096470791154,.006058545504236,
  .005014202742928,.003964554338446,.002910731817934,.001853960788946,
  .000796792065552};
  dX=(X2-X1)/2;
  aX=(X1+X2)/2;
  G=0;
  for(i=0;i<48;i++)
    G+=W[i]*(F(aX-dX*A[i])+F(aX+dX*A[i]));
  G*=dX;
  return(G);
  }

void test_Trapz(char*name,int n,double FUN(double),double X1,double X2)
  {                                                  /* test Trapezoidal Rule */
  long i,j;
  double E,F,R,T;
  printf("%s",name);
  QueryPerformanceCounter(&t1);
  j=0;
  do{
    for(i=0;i<100;i++)
      F=Trapz(n,FUN,X1,X2);
    j+=i;
    QueryPerformanceCounter(&t2);
    }while(t2<t1+rate);
  E=fabs(F-log(X2/X1));
  T=(t2-t1)/((double)rate)/j;
  R=log10(1./E)/T;
  printf(" %18.15lf %17.15lf %12.10lf %7.0lf\n",F,E,T,R);
  }

void test_Simps(char*name,int n,double FUN(double),double X1,double X2)
  {                                                    /* test Simpson's Rule */
  int i,j;
  double E,F,R,T;
  printf("%s",name);
  QueryPerformanceCounter(&t1);
  j=0;
  do{
    for(i=0;i<100;i++)
      F=Simps(FUN,X1,X2,n);
    j+=i;
    QueryPerformanceCounter(&t2);
    }while(t2<t1+rate);
  E=fabs(F-log(X2/X1));
  T=(t2-t1)/((double)rate)/j;
  R=log10(1./E)/T;
  printf(" %18.15lf %17.15lf %12.10lf %7.0lf\n",F,E,T,R);
  }

void test_Rombg(char*name,int n,double FUN(double),double X1,double X2)
  {                                                  /* test Romberg's Method */
  int i,j;
  double E,F,R,T;
  printf("%s",name);
  QueryPerformanceCounter(&t1);
  j=0;
  do{
    for(i=0;i<100;i++)
      F=Rombg(FUN,X1,X2,n);
    j+=i;
    QueryPerformanceCounter(&t2);
    }while(t2<t1+rate);
  E=fabs(F-log(X2/X1));
  T=(t2-t1)/((double)rate)/j;
  R=log10(1./E)/T;
  printf(" %18.15lf %17.15lf %12.10lf %7.0lf\n",F,E,T,R);
  }

void test_other(char*name,double MET(double fun(double),double,double),int n,double FUN(double),double X1,double X2)
  {                                                     /* test other methods */
  int i,j;
  double E,F,R,T;
  printf("%s",name);
  QueryPerformanceCounter(&t1);
  j=0;
  do{
    for(i=0;i<100;i++)
      F=MET(FUN,X1,X2);
    j+=i;
    QueryPerformanceCounter(&t2);
    }while(t2<t1+rate);
  E=fabs(F-log(X2/X1));
  T=(t2-t1)/((double)rate)/j;
  R=log10(1./E)/T;
  printf(" %18.15lf %17.15lf %12.10lf %7.0lf\n",F,E,T,R);
  }

void test_subdv(char*name,double MET(double fun(double),double,double),int n,int l,double FUN(double),double X1,double X2)
  {                                                  /* test subdivided Gauss */
  int i,j,k;
  double E,F,R,T;
  printf("%s",name);
  QueryPerformanceCounter(&t1);
  j=0;
  do{
    for(i=0;i<100;i++)
      for(F=k=0;k<l;k++)
        F+=MET(FUN,X1+k*(X2-X1)/l,X1+(k+1)*(X2-X1)/l);
    j+=i;
    QueryPerformanceCounter(&t2);
    }while(t2<t1+rate);
  E=fabs(F-log(X2/X1));
  T=(t2-t1)/((double)rate)/j;
  R=log10(1./E)/T;
  printf(" %18.15lf %17.15lf %12.10lf %7.0lf\n",F,E,T,R);
  }

/*******************************************************************************
  Benton's method is 5-Point Gauss Quadrature applied over n subdivisions
  of the total interval. The relative sizes of the subdivisions are
  inversely proportional to the change in the function over that interval.
  In other words, the integration focuses on the segments where the function
  changes most rapidly. This is a special type of subdivided Gauss Quadrature.
*******************************************************************************/

double Bento(int n,double FUN(double),double X1,double X2)
  {                                                   /* test Benton's method */
  #define N 10
  int i;
  double B,C,D[N],F[N+1],X;
  if(n<2||n>N)
    abort();
  for(i=0;i<=n;i++)
    F[i]=FUN(X1+i*(X2-X1)/n);
  B=0;
  for(i=0;i<n;i++)
    {
    D[i]=fabs(F[i+1]-F[i]);
    B+=D[i];
    }
  if(B<=DBL_EPSILON)
    return((X2-X1)*F[0]);
  C=B/100;
  B=0;
  for(i=0;i<n;i++)
    {
    D[i]=1./max(C,D[i]);
    B+=D[i];
    }
  X=X1;
  B/=(X2-X1);
  C=0;
  for(i=0;i<n;i++)
    {
    X+=D[i]/B;
    C+=Gaus5(FUN,X1,X);
    X1=X;
    }
  return(C);
  }

void test_Bento(char*name,int n,double FUN(double),double X1,double X2)
  {
  int i,j;
  double E,F,R,T;
  printf("%s",name);
  QueryPerformanceCounter(&t1);
  j=0;
  do{
    for(i=0;i<100;i++)
      F=Bento(n,FUN,X1,X2);
    j+=i;
    QueryPerformanceCounter(&t2);
    }while(t2<t1+rate);
  E=fabs(F-log(X2/X1));
  T=(t2-t1)/((double)rate)/j;
  R=log10(1./E)/T;
  printf(" %18.15lf %17.15lf %12.10lf %7.0lf\n",F,E,T,R);
  }

int main(int argc,char**argv,char**envp)
  {
  double X1=.1,X2=1;

  printf("INTEST/V%s: test methods for numerical integration\n",Version);
  printf("method                      result             error           sec     dig/sec\n");
  printf("exact=ln(10)        ");
  printf(" %18.15lf",log(X2/X1));
  printf("         N/A            N/A       N/A\n");
  QueryPerformanceFrequency(&rate);

  test_Trapz("3-point trapezoidal ",    3,Ftest,X1,X2);
  test_Trapz("5-point trapezoidal ",    5,Ftest,X1,X2);
  test_Trapz("10-point trapezoidal",   10,Ftest,X1,X2);
  test_Trapz("20-point trapezoidal",   20,Ftest,X1,X2);
  test_Trapz("100-point trapezoida",  100,Ftest,X1,X2);
  test_Trapz("1000-point trapezoid", 1000,Ftest,X1,X2);
  test_Trapz("10000-point trapezoi",10000,Ftest,X1,X2);

  test_Simps("3-point Simpson     ",    3,Ftest,X1,X2);
  test_Simps("5-point Simpson     ",    5,Ftest,X1,X2);
  test_Simps("10-point Simpson    ",   10,Ftest,X1,X2);
  test_Simps("20-point Simpson    ",   20,Ftest,X1,X2);
  test_Simps("100-point Simpson   ",  100,Ftest,X1,X2);
  test_Simps("1000-point Simpson  ", 1000,Ftest,X1,X2);
  test_Simps("10000-point Simpson ",10000,Ftest,X1,X2);

  test_other("4-point Newton-Cotes",NewC4 , 4,Ftest,X1,X2);
  test_other("5-point Newton-Cotes",NewC5 , 5,Ftest,X1,X2);
  test_other("6-point Newton-Cotes",NewC6 , 6,Ftest,X1,X2);
  test_other("7-point Newton-Cotes",NewC7 , 7,Ftest,X1,X2);
  test_other("8-point Newton-Cotes",NewC8 , 8,Ftest,X1,X2);
  test_other("9-point Newton-Cotes",NewC9 , 9,Ftest,X1,X2);
  test_other("10-point Newton-Cote",NewC10,10,Ftest,X1,X2);
  test_other("11-point Newton-Cote",NewC11,11,Ftest,X1,X2);
  test_other("12-point Newton-Cote",NewC12,12,Ftest,X1,X2);
  test_other("16-point Newton-Cote",NewC16,16,Ftest,X1,X2);
  test_other("20-point Newton-Cote",NewC20,20,Ftest,X1,X2);
  test_other("30-point Newton-Cote",NewC30,30,Ftest,X1,X2);
  test_other("40-point Newton-Cote",NewC40,40,Ftest,X1,X2);
  test_other("50-point Newton-Cote",NewC50,50,Ftest,X1,X2);
  test_other("60-point Newton-Cote",NewC60,60,Ftest,X1,X2);

  test_Rombg("3-point Romberg     ", 2,Ftest,X1,X2);
  test_Rombg("5-point Romberg     ", 3,Ftest,X1,X2);
  test_Rombg("9-point Romberg     ", 4,Ftest,X1,X2);
  test_Rombg("17-point Romberg    ", 5,Ftest,X1,X2);
  test_Rombg("33-point Romberg    ", 6,Ftest,X1,X2);
  test_Rombg("65-point Romberg    ", 7,Ftest,X1,X2);
  test_Rombg("129-point Romberg   ", 8,Ftest,X1,X2);
  test_Rombg("257-point Romberg   ", 9,Ftest,X1,X2);
  test_Rombg("513-point Romberg   ",10,Ftest,X1,X2);
  test_Rombg("1025-point Romberg  ",11,Ftest,X1,X2);
  test_Rombg("2049-point Romberg  ",12,Ftest,X1,X2);
  test_Rombg("4097-point Romberg  ",13,Ftest,X1,X2);
  test_Rombg("8193-point Romberg  ",14,Ftest,X1,X2);
  test_Rombg("16385-point Romberg ",15,Ftest,X1,X2);

  test_other("3-point Chebyshev   ",Cheb3,3,Ftest,X1,X2);
  test_other("4-point Chebyshev   ",Cheb4,4,Ftest,X1,X2);
  test_other("5-point Chebyshev   ",Cheb5,5,Ftest,X1,X2);
  test_other("6-point Chebyshev   ",Cheb6,6,Ftest,X1,X2);
  test_other("7-point Chebyshev   ",Cheb7,7,Ftest,X1,X2);
  test_other("9-point Chebyshev   ",Cheb9,8,Ftest,X1,X2);

  test_other("3-point Lobatto     ",Loba3 , 3,Ftest,X1,X2);
  test_other("4-point Lobatto     ",Loba4 , 4,Ftest,X1,X2);
  test_other("5-point Lobatto     ",Loba5 , 5,Ftest,X1,X2);
  test_other("6-point Lobatto     ",Loba6 , 6,Ftest,X1,X2);
  test_other("7-point Lobatto     ",Loba7 , 7,Ftest,X1,X2);
  test_other("8-point Lobatto     ",Loba8 , 8,Ftest,X1,X2);
  test_other("9-point Lobatto     ",Loba9 , 9,Ftest,X1,X2);
  test_other("10-point Lobatto    ",Loba10,10,Ftest,X1,X2);
  test_other("12-point Lobatto    ",Loba12,12,Ftest,X1,X2);
  test_other("16-point Lobatto    ",Loba16,16,Ftest,X1,X2);
  test_other("20-point Lobatto    ",Loba20,20,Ftest,X1,X2);
  test_other("24-point Lobatto    ",Loba24,24,Ftest,X1,X2);
  test_other("32-point Lobatto    ",Loba32,32,Ftest,X1,X2);
  test_other("48-point Lobatto    ",Loba48,48,Ftest,X1,X2);
  test_other("64-point Lobatto    ",Loba64,64,Ftest,X1,X2);
  test_other("96-point Lobatto    ",Loba96,96,Ftest,X1,X2);

  test_other("2-point Gauss       ",Gaus2 , 2,Ftest,X1,X2);
  test_other("3-point Gauss       ",Gaus3 , 3,Ftest,X1,X2);
  test_other("4-point Gauss       ",Gaus4 , 4,Ftest,X1,X2);
  test_other("5-point Gauss       ",Gaus5 , 5,Ftest,X1,X2);
  test_other("6-point Gauss       ",Gaus6 , 6,Ftest,X1,X2);
  test_other("7-point Gauss       ",Gaus7 , 7,Ftest,X1,X2);
  test_other("8-point Gauss       ",Gaus8 , 8,Ftest,X1,X2);
  test_other("9-point Gauss       ",Gaus9 , 9,Ftest,X1,X2);
  test_other("10-point Gauss      ",Gaus10,10,Ftest,X1,X2);
  test_other("12-point Gauss      ",Gaus12,12,Ftest,X1,X2);
  test_other("16-point Gauss      ",Gaus16,16,Ftest,X1,X2);
  test_other("20-point Gauss      ",Gaus20,20,Ftest,X1,X2);
  test_other("40-point Gauss      ",Gaus40,40,Ftest,X1,X2);
  test_other("96-point Gauss      ",Gaus96,96,Ftest,X1,X2);

  test_subdv("2*5-point Gauss     ",Gaus5,5,  2,Ftest,X1,X2);
  test_subdv("10*5-point Gauss    ",Gaus5,5, 10,Ftest,X1,X2);
  test_subdv("20*5-point Gauss    ",Gaus5,5, 20,Ftest,X1,X2);
  test_subdv("100*5-point Gauss   ",Gaus5,5,100,Ftest,X1,X2);

  test_Bento("10-point Benton     ", 2,Ftest,X1,X2);
  test_Bento("15-point Benton     ", 3,Ftest,X1,X2);
  test_Bento("20-point Benton     ", 4,Ftest,X1,X2);
  test_Bento("25-point Benton     ", 5,Ftest,X1,X2);
  test_Bento("30-point Benton     ", 6,Ftest,X1,X2);
  test_Bento("35-point Benton     ", 7,Ftest,X1,X2);
  test_Bento("40-point Benton     ", 8,Ftest,X1,X2);
  test_Bento("45-point Benton     ", 9,Ftest,X1,X2);
  test_Bento("50-point Benton     ",10,Ftest,X1,X2);

  return(0);
  }
